.qq-uploader { position:relative; width: 100%;}

.qq-upload-button {
    display:block; /* or inline-block */
    width: 105px; padding: 7px; text-align:center;
    background:#880000; border-bottom:1px solid #ddd;color:#fff;
    float: right;
    cursor: pointer !important;
    margin-top: 10px;
    -moz-border-radius:5px;
    -webkit-border-radius:5px;
    border-radius:5px;
}

.qq-upload-button input[type="file"],
.qq-upload-button input[type="file"]:hover {cursor: pointer !important;}
.qq-upload-button-hover {background:#cc0000;cursor: pointer !important;}
.qq-upload-button-focus {outline:1px dotted black;cursor: pointer !important;}

.qq-upload-drop-area {
    position:absolute; top:0; left:0; width:100%; height:100%; min-height: 70px; z-index:2;
    background: #F5F5F5; text-align:center; border: 1px dotted black;
}
.qq-upload-drop-area span {
    display:block; position:absolute; top: 73%; width:100%; font-size:12px;
}
.qq-upload-drop-area-active {}

.qq-upload-list {display: block; margin:15px 0; padding:0; list-style:disc; clear: both; list-style: none outside none}

.qq-upload-list li {
    clear: both;
    display: block;
    list-style: none outside none;
    background-color:#F5F5F5;
    border:3px solid #E5E5E5;
    margin:5px 0 0 0;
    padding:5px 10px;
    font-size: 11px;
}

.qq-upload-file {}
.qq-upload-spinner {display:none; background: url("++resource++quickupload_static/loading.gif"); width:15px; height:15px; vertical-align:text-bottom; margin-left: 10px}
.qq-upload-size,.qq-upload-cancel {font-size:11px;}

.qq-upload-list .qq-upload-cancel {
   display: block;
   float: right;
   width: 16px;
   height: 16px;
   background: transparent url("++resource++quickupload_static/cancel.png") top left no-repeat;
}



.qq-upload-fail { background-color: #FDE5DD !important; }
.qq-upload-failed-text {display:none; margin-left: 10px;}
.qq-upload-fail .qq-upload-failed-text {display:inline; color: red;}


.qq-upload-list li .qq-upload-size {
    clear: both;
    min-height: 17px;
}

.qq-upload-list li .sizeContainer {
    background-color: #FFFFFF;
    border-top: 1px solid #808080;
    border-left: 1px solid #808080;
    border-right: 1px solid #C5C5C5;
    border-bottom: 1px solid #C5C5C5;
    margin-top: 6px;
    width: 75%;
    height: 3px;
    float: left;
}
.qq-upload-list li .sizeContainer .sizeBar {
    background-color: #0099FF;
    width: 2px;
    height: 3px;
}

.qq-upload-fail .sizeBar { background-color: red !important; }

.qq-upload-list li .sizeTotal {
   width: auto;
   float: right;
   text-align: right;
}

.qq-upload-infos {
   clear: both;
}

.qq-upload-list li .server-error {
    color: #880000;
    clear: both;
}

/* IE7 fixes */

*+html .quick-uploader #label-upload {
    float: none !important;
}

*+html .quick-uploader .qq-upload-button {
    height: 20px !important;
    padding: 4px !important;
}

*+html .quick-uploader .qq-upload-button input[type="file"] {
    font-size: 23px !important;
}
